@@ -31,15 +31,15 @@ static const char *manufacturers[] = {
 	"Rohde&Schwarz",
 };
 
-static const uint32_t drvopts[] = {
-	SR_CONF_OSCILLOSCOPE,
-};
-
 static const uint32_t scanopts[] = {
 	SR_CONF_CONN,
 	SR_CONF_SERIALCOMM,
 };
 
+static const uint32_t drvopts[] = {
+	SR_CONF_OSCILLOSCOPE,
+};
+
 enum {
 	CG_INVALID = -1,
 	CG_NONE,
@@ -47,18 +47,7 @@ enum {
 	CG_DIGITAL,
 };
 
-static int check_manufacturer(const char *manufacturer)
-{
-	unsigned int i;
-
-	for (i = 0; i < ARRAY_SIZE(manufacturers); i++)
-		if (!strcmp(manufacturer, manufacturers[i]))
-			return SR_OK;
-
-	return SR_ERR;
-}
-
-static struct sr_dev_inst *hmo_probe_serial_device(struct sr_scpi_dev_inst *scpi)
+static struct sr_dev_inst *probe_device(struct sr_scpi_dev_inst *scpi)
 {
 	struct sr_dev_inst *sdi;
 	struct dev_context *devc;
@@ -73,7 +62,7 @@ static struct sr_dev_inst *hmo_probe_serial_device(struct sr_scpi_dev_inst *scpi
 		goto fail;
 	}
 
-	if (check_manufacturer(hw_info->manufacturer) != SR_OK)
+	if (std_str_idx_s(hw_info->manufacturer, ARRAY_AND_SIZE(manufacturers)) < 0)
 		goto fail;
 
 	sdi = g_malloc0(sizeof(struct sr_dev_inst));
@@ -107,57 +96,40 @@ fail:
 
 static GSList *scan(struct sr_dev_driver *di, GSList *options)
 {
-	return sr_scpi_scan(di->context, options, hmo_probe_serial_device);
+	return sr_scpi_scan(di->context, options, probe_device);
 }
 
-static void clear_helper(void *priv)
+static void clear_helper(struct dev_context *devc)
 {
-	struct dev_context *devc;
-
-	devc = priv;
-
 	hmo_scope_state_free(devc->model_state);
-
 	g_free(devc->analog_groups);
 	g_free(devc->digital_groups);
-
-	g_free(devc);
 }
 
 static int dev_clear(const struct sr_dev_driver *di)
 {
-	return std_dev_clear(di, clear_helper);
+	return std_dev_clear_with_callback(di, (std_dev_clear_callback)clear_helper);
 }
 
 static int dev_open(struct sr_dev_inst *sdi)
 {
-	if (sdi->status != SR_ST_ACTIVE && sr_scpi_open(sdi->conn) != SR_OK)
+	if (sr_scpi_open(sdi->conn) != SR_OK)
 		return SR_ERR;
 
 	if (hmo_scope_state_get(sdi) != SR_OK)
 		return SR_ERR;
 
-	sdi->status = SR_ST_ACTIVE;
-
 	return SR_OK;
 }
 
 static int dev_close(struct sr_dev_inst *sdi)
 {
-	if (sdi->status == SR_ST_INACTIVE)
-		return SR_OK;
-
-	sr_scpi_close(sdi->conn);
-
-	sdi->status = SR_ST_INACTIVE;
-
-	return SR_OK;
+	return sr_scpi_close(sdi->conn);
 }
 
 static int check_channel_group(struct dev_context *devc,
 			     const struct sr_channel_group *cg)
 {
-	unsigned int i;
 	const struct scope_config *model;
 
 	model = devc->model_config;
@@ -165,24 +137,21 @@ static int check_channel_group(struct dev_context *devc,
 	if (!cg)
 		return CG_NONE;
 
-	for (i = 0; i < model->analog_channels; i++)
-		if (cg == devc->analog_groups[i])
-			return CG_ANALOG;
+	if (std_cg_idx(cg, devc->analog_groups, model->analog_channels) >= 0)
+		return CG_ANALOG;
 
-	for (i = 0; i < model->digital_pods; i++)
-		if (cg == devc->digital_groups[i])
-			return CG_DIGITAL;
+	if (std_cg_idx(cg, devc->digital_groups, model->digital_pods) >= 0)
+		return CG_DIGITAL;
 
 	sr_err("Invalid channel group specified.");
 
 	return CG_INVALID;
 }
 
-static int config_get(uint32_t key, GVariant **data, const struct sr_dev_inst *sdi,
-		      const struct sr_channel_group *cg)
+static int config_get(uint32_t key, GVariant **data,
+	const struct sr_dev_inst *sdi, const struct sr_channel_group *cg)
 {
-	int ret, cg_type;
-	unsigned int i;
+	int cg_type, idx;
 	struct dev_context *devc;
 	const struct scope_config *model;
 	struct scope_state *state;
@@ -195,126 +164,73 @@ static int config_get(uint32_t key, GVariant **data, const struct sr_dev_inst *s
 	if ((cg_type = check_channel_group(devc, cg)) == CG_INVALID)
 		return SR_ERR;
 
-	ret = SR_ERR_NA;
 	model = devc->model_config;
 	state = devc->model_state;
 
 	switch (key) {
 	case SR_CONF_NUM_HDIV:
 		*data = g_variant_new_int32(model->num_xdivs);
-		ret = SR_OK;
 		break;
 	case SR_CONF_TIMEBASE:
 		*data = g_variant_new("(tt)", (*model->timebases)[state->timebase][0],
 				      (*model->timebases)[state->timebase][1]);
-		ret = SR_OK;
 		break;
 	case SR_CONF_NUM_VDIV:
-		if (cg_type == CG_NONE) {
-			sr_err("No channel group specified.");
+		if (!cg)
 			return SR_ERR_CHANNEL_GROUP;
-		} else if (cg_type == CG_ANALOG) {
-			for (i = 0; i < model->analog_channels; i++) {
-				if (cg != devc->analog_groups[i])
-					continue;
-				*data = g_variant_new_int32(model->num_ydivs);
-				ret = SR_OK;
-				break;
-			}
-
-		} else {
-			ret = SR_ERR_NA;
-		}
+		if (cg_type != CG_ANALOG)
+			return SR_ERR_NA;
+		if (std_cg_idx(cg, devc->analog_groups, model->analog_channels) < 0)
+			return SR_ERR_ARG;
+		*data = g_variant_new_int32(model->num_ydivs);
 		break;
 	case SR_CONF_VDIV:
-		if (cg_type == CG_NONE) {
-			sr_err("No channel group specified.");
+		if (!cg)
 			return SR_ERR_CHANNEL_GROUP;
-		} else if (cg_type == CG_ANALOG) {
-			for (i = 0; i < model->analog_channels; i++) {
-				if (cg != devc->analog_groups[i])
-					continue;
-				*data = g_variant_new("(tt)",
-						      (*model->vdivs)[state->analog_channels[i].vdiv][0],
-						      (*model->vdivs)[state->analog_channels[i].vdiv][1]);
-				ret = SR_OK;
-				break;
-			}
-
-		} else {
-			ret = SR_ERR_NA;
-		}
+		if (cg_type != CG_ANALOG)
+			return SR_ERR_NA;
+		if ((idx = std_cg_idx(cg, devc->analog_groups, model->analog_channels)) < 0)
+			return SR_ERR_ARG;
+		*data = g_variant_new("(tt)",
+				      (*model->vdivs)[state->analog_channels[idx].vdiv][0],
+				      (*model->vdivs)[state->analog_channels[idx].vdiv][1]);
 		break;
 	case SR_CONF_TRIGGER_SOURCE:
 		*data = g_variant_new_string((*model->trigger_sources)[state->trigger_source]);
-		ret = SR_OK;
 		break;
 	case SR_CONF_TRIGGER_SLOPE:
 		*data = g_variant_new_string((*model->trigger_slopes)[state->trigger_slope]);
-		ret = SR_OK;
 		break;
 	case SR_CONF_HORIZ_TRIGGERPOS:
 		*data = g_variant_new_double(state->horiz_triggerpos);
-		ret = SR_OK;
 		break;
 	case SR_CONF_COUPLING:
-		if (cg_type == CG_NONE) {
-			sr_err("No channel group specified.");
+		if (!cg)
 			return SR_ERR_CHANNEL_GROUP;
-		} else if (cg_type == CG_ANALOG) {
-			for (i = 0; i < model->analog_channels; i++) {
-				if (cg != devc->analog_groups[i])
-					continue;
-				*data = g_variant_new_string((*model->coupling_options)[state->analog_channels[i].coupling]);
-				ret = SR_OK;
-				break;
-			}
-
-		} else {
-			ret = SR_ERR_NA;
-		}
+		if (cg_type != CG_ANALOG)
+			return SR_ERR_NA;
+		if ((idx = std_cg_idx(cg, devc->analog_groups, model->analog_channels)) < 0)
+			return SR_ERR_ARG;
+		*data = g_variant_new_string((*model->coupling_options)[state->analog_channels[idx].coupling]);
 		break;
 	case SR_CONF_SAMPLERATE:
 		*data = g_variant_new_uint64(state->sample_rate);
-		ret = SR_OK;
 		break;
 	default:
-		ret = SR_ERR_NA;
+		return SR_ERR_NA;
 	}
 
-	return ret;
+	return SR_OK;
 }
 
-static GVariant *build_tuples(const uint64_t (*array)[][2], unsigned int n)
+static int config_set(uint32_t key, GVariant *data,
+	const struct sr_dev_inst *sdi, const struct sr_channel_group *cg)
 {
-	unsigned int i;
-	GVariant *rational[2];
-	GVariantBuilder gvb;
-
-	g_variant_builder_init(&gvb, G_VARIANT_TYPE_ARRAY);
-
-	for (i = 0; i < n; i++) {
-		rational[0] = g_variant_new_uint64((*array)[i][0]);
-		rational[1] = g_variant_new_uint64((*array)[i][1]);
-
-		/* FIXME: Valgrind reports a memory leak here. */
-		g_variant_builder_add_value(&gvb, g_variant_new_tuple(rational, 2));
-	}
-
-	return g_variant_builder_end(&gvb);
-}
-
-static int config_set(uint32_t key, GVariant *data, const struct sr_dev_inst *sdi,
-		      const struct sr_channel_group *cg)
-{
-	int ret, cg_type;
-	unsigned int i, j;
+	int ret, cg_type, idx, j;
 	char command[MAX_COMMAND_SIZE], float_str[30];
 	struct dev_context *devc;
 	const struct scope_config *model;
 	struct scope_state *state;
-	const char *tmp;
-	uint64_t p, q;
 	double tmp_d;
 	gboolean update_sample_rate;
 
@@ -330,139 +246,89 @@ static int config_set(uint32_t key, GVariant *data, const struct sr_dev_inst *sd
 	state = devc->model_state;
 	update_sample_rate = FALSE;
 
-	ret = SR_ERR_NA;
-
 	switch (key) {
 	case SR_CONF_LIMIT_FRAMES:
 		devc->frame_limit = g_variant_get_uint64(data);
 		ret = SR_OK;
 		break;
 	case SR_CONF_TRIGGER_SOURCE:
-		tmp = g_variant_get_string(data, NULL);
-		for (i = 0; (*model->trigger_sources)[i]; i++) {
-			if (g_strcmp0(tmp, (*model->trigger_sources)[i]) != 0)
-				continue;
-			state->trigger_source = i;
-			g_snprintf(command, sizeof(command),
-				   (*model->scpi_dialect)[SCPI_CMD_SET_TRIGGER_SOURCE],
-				   (*model->trigger_sources)[i]);
-
-			ret = sr_scpi_send(sdi->conn, command);
-			break;
-		}
+		if ((idx = std_str_idx(data, *model->trigger_sources, model->num_trigger_sources)) < 0)
+			return SR_ERR_ARG;
+		state->trigger_source = idx;
+		g_snprintf(command, sizeof(command),
+			   (*model->scpi_dialect)[SCPI_CMD_SET_TRIGGER_SOURCE],
+			   (*model->trigger_sources)[idx]);
+		ret = sr_scpi_send(sdi->conn, command);
 		break;
 	case SR_CONF_VDIV:
-		if (cg_type == CG_NONE) {
-			sr_err("No channel group specified.");
+		if (!cg)
 			return SR_ERR_CHANNEL_GROUP;
-		}
-
-		g_variant_get(data, "(tt)", &p, &q);
-
-		for (i = 0; i < model->num_vdivs; i++) {
-			if (p != (*model->vdivs)[i][0] ||
-			    q != (*model->vdivs)[i][1])
-				continue;
-			for (j = 1; j <= model->analog_channels; j++) {
-				if (cg != devc->analog_groups[j - 1])
-					continue;
-				state->analog_channels[j - 1].vdiv = i;
-				g_ascii_formatd(float_str, sizeof(float_str), "%E", (float) p / q);
-				g_snprintf(command, sizeof(command),
-					   (*model->scpi_dialect)[SCPI_CMD_SET_VERTICAL_DIV],
-					   j, float_str);
-
-				if (sr_scpi_send(sdi->conn, command) != SR_OK ||
-				    sr_scpi_get_opc(sdi->conn) != SR_OK)
-					return SR_ERR;
-
-				break;
-			}
-
-			ret = SR_OK;
-			break;
-		}
+		if ((idx = std_u64_tuple_idx(data, *model->vdivs, model->num_vdivs)) < 0)
+			return SR_ERR_ARG;
+		if ((j = std_cg_idx(cg, devc->analog_groups, model->analog_channels)) < 0)
+			return SR_ERR_ARG;
+		state->analog_channels[j].vdiv = idx;
+		g_ascii_formatd(float_str, sizeof(float_str), "%E",
+			(float) (*model->vdivs)[idx][0] / (*model->vdivs)[idx][1]);
+		g_snprintf(command, sizeof(command),
+			   (*model->scpi_dialect)[SCPI_CMD_SET_VERTICAL_DIV],
+			   j + 1, float_str);
+		if (sr_scpi_send(sdi->conn, command) != SR_OK ||
+		    sr_scpi_get_opc(sdi->conn) != SR_OK)
+			return SR_ERR;
+		ret = SR_OK;
 		break;
 	case SR_CONF_TIMEBASE:
-		g_variant_get(data, "(tt)", &p, &q);
-
-		for (i = 0; i < model->num_timebases; i++) {
-			if (p != (*model->timebases)[i][0] ||
-			    q != (*model->timebases)[i][1])
-				continue;
-			state->timebase = i;
-			g_ascii_formatd(float_str, sizeof(float_str), "%E", (float) p / q);
-			g_snprintf(command, sizeof(command),
-				   (*model->scpi_dialect)[SCPI_CMD_SET_TIMEBASE],
-				   float_str);
-
-			ret = sr_scpi_send(sdi->conn, command);
-			update_sample_rate = TRUE;
-			break;
-		}
+		if ((idx = std_u64_tuple_idx(data, *model->timebases, model->num_timebases)) < 0)
+			return SR_ERR_ARG;
+		state->timebase = idx;
+		g_ascii_formatd(float_str, sizeof(float_str), "%E",
+			(float) (*model->timebases)[idx][0] / (*model->timebases)[idx][1]);
+		g_snprintf(command, sizeof(command),
+			   (*model->scpi_dialect)[SCPI_CMD_SET_TIMEBASE],
+			   float_str);
+		ret = sr_scpi_send(sdi->conn, command);
+		update_sample_rate = TRUE;
 		break;
 	case SR_CONF_HORIZ_TRIGGERPOS:
 		tmp_d = g_variant_get_double(data);
-
 		if (tmp_d < 0.0 || tmp_d > 1.0)
 			return SR_ERR;
-
 		state->horiz_triggerpos = tmp_d;
 		tmp_d = -(tmp_d - 0.5) *
 			((double) (*model->timebases)[state->timebase][0] /
 			(*model->timebases)[state->timebase][1])
 			 * model->num_xdivs;
-
 		g_ascii_formatd(float_str, sizeof(float_str), "%E", tmp_d);
 		g_snprintf(command, sizeof(command),
 			   (*model->scpi_dialect)[SCPI_CMD_SET_HORIZ_TRIGGERPOS],
 			   float_str);
-
 		ret = sr_scpi_send(sdi->conn, command);
 		break;
 	case SR_CONF_TRIGGER_SLOPE:
-		tmp = g_variant_get_string(data, NULL);
-		for (i = 0; (*model->trigger_slopes)[i]; i++) {
-			if (g_strcmp0(tmp, (*model->trigger_slopes)[i]) != 0)
-				continue;
-			state->trigger_slope = i;
-			g_snprintf(command, sizeof(command),
-				   (*model->scpi_dialect)[SCPI_CMD_SET_TRIGGER_SLOPE],
-				   (*model->trigger_slopes)[i]);
-
-			ret = sr_scpi_send(sdi->conn, command);
-			break;
-		}
+		if ((idx = std_str_idx(data, *model->trigger_slopes, model->num_trigger_slopes)) < 0)
+			return SR_ERR_ARG;
+		state->trigger_slope = idx;
+		g_snprintf(command, sizeof(command),
+			   (*model->scpi_dialect)[SCPI_CMD_SET_TRIGGER_SLOPE],
+			   (*model->trigger_slopes)[idx]);
+		ret = sr_scpi_send(sdi->conn, command);
 		break;
 	case SR_CONF_COUPLING:
-		if (cg_type == CG_NONE) {
-			sr_err("No channel group specified.");
+		if (!cg)
 			return SR_ERR_CHANNEL_GROUP;
-		}
-
-		tmp = g_variant_get_string(data, NULL);
-
-		for (i = 0; (*model->coupling_options)[i]; i++) {
-			if (strcmp(tmp, (*model->coupling_options)[i]) != 0)
-				continue;
-			for (j = 1; j <= model->analog_channels; j++) {
-				if (cg != devc->analog_groups[j - 1])
-					continue;
-				state->analog_channels[j-1].coupling = i;
-
-				g_snprintf(command, sizeof(command),
-					   (*model->scpi_dialect)[SCPI_CMD_SET_COUPLING],
-					   j, tmp);
-
-				if (sr_scpi_send(sdi->conn, command) != SR_OK ||
-				    sr_scpi_get_opc(sdi->conn) != SR_OK)
-					return SR_ERR;
-				break;
-			}
-
-			ret = SR_OK;
-			break;
-		}
+		if ((idx = std_str_idx(data, *model->coupling_options, model->num_coupling_options)) < 0)
+			return SR_ERR_ARG;
+		if ((j = std_cg_idx(cg, devc->analog_groups, model->analog_channels)) < 0)
+			return SR_ERR_ARG;
+		state->analog_channels[j].coupling = idx;
+		g_snprintf(command, sizeof(command),
+			   (*model->scpi_dialect)[SCPI_CMD_SET_COUPLING],
+			   j + 1, (*model->coupling_options)[idx]);
+		if (sr_scpi_send(sdi->conn, command) != SR_OK ||
+		    sr_scpi_get_opc(sdi->conn) != SR_OK)
+			return SR_ERR;
+		ret = SR_OK;
 		break;
 	default:
 		ret = SR_ERR_NA;
@@ -478,8 +344,8 @@ static int config_set(uint32_t key, GVariant *data, const struct sr_dev_inst *sd
 	return ret;
 }
 
-static int config_list(uint32_t key, GVariant **data, const struct sr_dev_inst *sdi,
-		       const struct sr_channel_group *cg)
+static int config_list(uint32_t key, GVariant **data,
+	const struct sr_dev_inst *sdi, const struct sr_channel_group *cg)
 {
 	int cg_type = CG_NONE;
 	struct dev_context *devc = NULL;
@@ -495,53 +361,48 @@ static int config_list(uint32_t key, GVariant **data, const struct sr_dev_inst *
 
 	switch (key) {
 	case SR_CONF_SCAN_OPTIONS:
-		*data = g_variant_new_fixed_array(G_VARIANT_TYPE_UINT32,
-				scanopts, ARRAY_SIZE(scanopts), sizeof(uint32_t));
+		*data = std_gvar_array_u32(ARRAY_AND_SIZE(scanopts));
 		break;
 	case SR_CONF_DEVICE_OPTIONS:
-		if (cg_type == CG_NONE) {
+		if (!cg) {
 			if (model)
-				*data = g_variant_new_fixed_array(G_VARIANT_TYPE_UINT32,
-					model->devopts, model->num_devopts, sizeof(uint32_t));
+				*data = std_gvar_array_u32(*model->devopts, model->num_devopts);
 			else
-				*data = g_variant_new_fixed_array(G_VARIANT_TYPE_UINT32,
-					drvopts, ARRAY_SIZE(drvopts), sizeof(uint32_t));
+				*data = std_gvar_array_u32(ARRAY_AND_SIZE(drvopts));
 		} else if (cg_type == CG_ANALOG) {
-			*data = g_variant_new_fixed_array(G_VARIANT_TYPE_UINT32,
-				model->analog_devopts, model->num_analog_devopts,
-				sizeof(uint32_t));
+			*data = std_gvar_array_u32(*model->devopts_cg_analog, model->num_devopts_cg_analog);
 		} else {
-			*data = g_variant_new_fixed_array(G_VARIANT_TYPE_UINT32,
-				NULL, 0, sizeof(uint32_t));
+			*data = std_gvar_array_u32(NULL, 0);
 		}
 		break;
 	case SR_CONF_COUPLING:
-		if (cg_type == CG_NONE)
+		if (!cg)
 			return SR_ERR_CHANNEL_GROUP;
-		*data = g_variant_new_strv(*model->coupling_options,
-			   g_strv_length((char **)*model->coupling_options));
+		if (!model)
+			return SR_ERR_ARG;
+		*data = g_variant_new_strv(*model->coupling_options, model->num_coupling_options);
 		break;
 	case SR_CONF_TRIGGER_SOURCE:
 		if (!model)
 			return SR_ERR_ARG;
-		*data = g_variant_new_strv(*model->trigger_sources,
-			   g_strv_length((char **)*model->trigger_sources));
+		*data = g_variant_new_strv(*model->trigger_sources, model->num_trigger_sources);
 		break;
 	case SR_CONF_TRIGGER_SLOPE:
 		if (!model)
 			return SR_ERR_ARG;
-		*data = g_variant_new_strv(*model->trigger_slopes,
-			   g_strv_length((char **)*model->trigger_slopes));
+		*data = g_variant_new_strv(*model->trigger_slopes, model->num_trigger_slopes);
 		break;
 	case SR_CONF_TIMEBASE:
 		if (!model)
 			return SR_ERR_ARG;
-		*data = build_tuples(model->timebases, model->num_timebases);
+		*data = std_gvar_tuple_array(*model->timebases, model->num_timebases);
 		break;
 	case SR_CONF_VDIV:
-		if (cg_type == CG_NONE)
+		if (!cg)
 			return SR_ERR_CHANNEL_GROUP;
-		*data = build_tuples(model->vdivs, model->num_vdivs);
+		if (!model)
+			return SR_ERR_ARG;
+		*data = std_gvar_tuple_array(*model->vdivs, model->num_vdivs);
 		break;
 	default:
 		return SR_ERR_NA;
@@ -648,6 +509,7 @@ static int hmo_setup_channels(const struct sr_dev_inst *sdi)
 	struct sr_channel *ch;
 	struct dev_context *devc;
 	struct sr_scpi_dev_inst *scpi;
+	int ret;
 
 	devc = sdi->priv;
 	scpi = sdi->conn;
@@ -667,8 +529,10 @@ static int hmo_setup_channels(const struct sr_dev_inst *sdi)
 				   (*model->scpi_dialect)[SCPI_CMD_SET_ANALOG_CHAN_STATE],
 				   ch->index + 1, ch->enabled);
 
-			if (sr_scpi_send(scpi, command) != SR_OK)
+			if (sr_scpi_send(scpi, command) != SR_OK) {
+				g_free(pod_enabled);
 				return SR_ERR;
+			}
 			state->analog_channels[ch->index].state = ch->enabled;
 			setup_changed = TRUE;
 			break;
@@ -686,30 +550,37 @@ static int hmo_setup_channels(const struct sr_dev_inst *sdi)
 				   (*model->scpi_dialect)[SCPI_CMD_SET_DIG_CHAN_STATE],
 				   ch->index, ch->enabled);
 
-			if (sr_scpi_send(scpi, command) != SR_OK)
+			if (sr_scpi_send(scpi, command) != SR_OK) {
+				g_free(pod_enabled);
 				return SR_ERR;
+			}
 
 			state->digital_channels[ch->index] = ch->enabled;
 			setup_changed = TRUE;
 			break;
 		default:
+			g_free(pod_enabled);
 			return SR_ERR;
 		}
 	}
 
-	for (i = 1; i <= model->digital_pods; i++) {
-		if (state->digital_pods[i - 1] == pod_enabled[i - 1])
+	ret = SR_OK;
+	for (i = 0; i < model->digital_pods; i++) {
+		if (state->digital_pods[i] == pod_enabled[i])
 			continue;
 		g_snprintf(command, sizeof(command),
 			   (*model->scpi_dialect)[SCPI_CMD_SET_DIG_POD_STATE],
-			   i, pod_enabled[i - 1]);
-		if (sr_scpi_send(scpi, command) != SR_OK)
-			return SR_ERR;
-		state->digital_pods[i - 1] = pod_enabled[i - 1];
+			   i + 1, pod_enabled[i]);
+		if (sr_scpi_send(scpi, command) != SR_OK) {
+			ret = SR_ERR;
+			break;
+		}
+		state->digital_pods[i] = pod_enabled[i];
 		setup_changed = TRUE;
 	}
-
 	g_free(pod_enabled);
+	if (ret != SR_OK)
+		return ret;
 
 	if (setup_changed && hmo_update_sample_rate(sdi) != SR_OK)
 		return SR_ERR;
@@ -727,9 +598,6 @@ static int dev_acquisition_start(const struct sr_dev_inst *sdi)
 	struct sr_scpi_dev_inst *scpi;
 	int ret;
 
-	if (sdi->status != SR_ST_ACTIVE)
-		return SR_ERR_DEV_CLOSED;
-
 	scpi = sdi->conn;
 	devc = sdi->priv;
 
@@ -811,9 +679,6 @@ static int dev_acquisition_stop(struct sr_dev_inst *sdi)
 
 	std_session_send_df_end(sdi);
 
-	if (sdi->status != SR_ST_ACTIVE)
-		return SR_ERR_DEV_CLOSED;
-
 	devc = sdi->priv;
 
 	devc->num_frames = 0;
