@@ -2,6 +2,7 @@
  * This file is part of the libsigrok project.
  *
  * Copyright (C) 2014 Bert Vermeulen <bert@biot.com>
+ * Copyright (C) 2017 Frank Stettner <frank-stettner@gmx.net>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,6 +100,7 @@ static struct sr_dev_inst *probe_device(struct sr_scpi_dev_inst *scpi,
 
 	devc = g_malloc0(sizeof(struct dev_context));
 	devc->device = device;
+	sr_sw_limits_init(&devc->limits);
 	sdi->priv = devc;
 
 	if (device->num_channels) {
@@ -127,7 +129,7 @@ static struct sr_dev_inst *probe_device(struct sr_scpi_dev_inst *scpi,
 	for (ch_num = 0; ch_num < num_channels; ch_num++) {
 		/* Create one channel per measurable output unit. */
 		for (i = 0; i < ARRAY_SIZE(pci); i++) {
-			if (!scpi_cmd_get(devc->device->commands, pci[i].command))
+			if (!sr_scpi_cmd_get(devc->device->commands, pci[i].command))
 				continue;
 			g_snprintf(ch_name, 16, "%s%s", pci[i].prefix,
 					channels[ch_num].name);
@@ -164,7 +166,7 @@ static struct sr_dev_inst *probe_device(struct sr_scpi_dev_inst *scpi,
 	sr_scpi_hw_info_free(hw_info);
 	hw_info = NULL;
 
-	scpi_cmd(sdi, devc->device->commands, SCPI_CMD_LOCAL);
+	sr_scpi_cmd(sdi, devc->device->commands, 0, NULL, SCPI_CMD_LOCAL);
 
 	return sdi;
 }
@@ -251,23 +253,19 @@ static int dev_open(struct sr_dev_inst *sdi)
 	struct sr_scpi_dev_inst *scpi;
 	GVariant *beeper;
 
-	if (sdi->status != SR_ST_INACTIVE)
-		return SR_ERR;
-
 	scpi = sdi->conn;
 	if (sr_scpi_open(scpi) < 0)
 		return SR_ERR;
 
-	sdi->status = SR_ST_ACTIVE;
-
 	devc = sdi->priv;
-	scpi_cmd(sdi, devc->device->commands, SCPI_CMD_REMOTE);
+	sr_scpi_cmd(sdi, devc->device->commands, 0, NULL, SCPI_CMD_REMOTE);
 	devc->beeper_was_set = FALSE;
-	if (scpi_cmd_resp(sdi, devc->device->commands, &beeper,
-			G_VARIANT_TYPE_BOOLEAN, SCPI_CMD_BEEPER) == SR_OK) {
+	if (sr_scpi_cmd_resp(sdi, devc->device->commands, 0, NULL,
+			&beeper, G_VARIANT_TYPE_BOOLEAN, SCPI_CMD_BEEPER) == SR_OK) {
 		if (g_variant_get_boolean(beeper)) {
 			devc->beeper_was_set = TRUE;
-			scpi_cmd(sdi, devc->device->commands, SCPI_CMD_BEEPER_DISABLE);
+			sr_scpi_cmd(sdi, devc->device->commands,
+				0, NULL, SCPI_CMD_BEEPER_DISABLE);
 		}
 		g_variant_unref(beeper);
 	}
@@ -280,43 +278,39 @@ static int dev_close(struct sr_dev_inst *sdi)
 	struct sr_scpi_dev_inst *scpi;
 	struct dev_context *devc;
 
-	if (sdi->status != SR_ST_ACTIVE)
-		return SR_ERR_DEV_CLOSED;
-
 	devc = sdi->priv;
 	scpi = sdi->conn;
-	if (scpi) {
-		if (devc->beeper_was_set)
-			scpi_cmd(sdi, devc->device->commands, SCPI_CMD_BEEPER_ENABLE);
-		scpi_cmd(sdi, devc->device->commands, SCPI_CMD_LOCAL);
-		sr_scpi_close(scpi);
-		sdi->status = SR_ST_INACTIVE;
-	}
 
-	return SR_OK;
+	if (!scpi)
+		return SR_ERR_BUG;
+
+	if (devc->beeper_was_set)
+		sr_scpi_cmd(sdi, devc->device->commands,
+			0, NULL, SCPI_CMD_BEEPER_ENABLE);
+	sr_scpi_cmd(sdi, devc->device->commands, 0, NULL, SCPI_CMD_LOCAL);
+
+	return sr_scpi_close(scpi);
 }
 
-static void clear_helper(void *priv)
+static void clear_helper(struct dev_context *devc)
 {
-	struct dev_context *devc;
-
-	devc = priv;
 	g_free(devc->channels);
 	g_free(devc->channel_groups);
-	g_free(devc);
 }
 
 static int dev_clear(const struct sr_dev_driver *di)
 {
-	return std_dev_clear(di, clear_helper);
+	return std_dev_clear_with_callback(di, (std_dev_clear_callback)clear_helper);
 }
 
-static int config_get(uint32_t key, GVariant **data, const struct sr_dev_inst *sdi,
-		const struct sr_channel_group *cg)
+static int config_get(uint32_t key, GVariant **data,
+	const struct sr_dev_inst *sdi, const struct sr_channel_group *cg)
 {
 	struct dev_context *devc;
 	const GVariantType *gvtype;
 	unsigned int i;
+	int channel_group_cmd;
+	char *channel_group_name;
 	int cmd, ret;
 	const char *s;
 
@@ -407,13 +401,23 @@ static int config_get(uint32_t key, GVariant **data, const struct sr_dev_inst *s
 	case SR_CONF_REGULATION:
 		gvtype = G_VARIANT_TYPE_STRING;
 		cmd = SCPI_CMD_GET_OUTPUT_REGULATION;
+		break;
+	default:
+		return sr_sw_limits_config_get(&devc->limits, key, data);
 	}
 	if (!gvtype)
 		return SR_ERR_NA;
 
-	if (cg)
-		select_channel(sdi, cg->channels->data);
-	ret = scpi_cmd_resp(sdi, devc->device->commands, data, gvtype, cmd);
+	channel_group_cmd = 0;
+	channel_group_name = NULL;
+	if (cg) {
+		channel_group_cmd = SCPI_CMD_SELECT_CHANNEL;
+		channel_group_name = g_strdup(cg->name);
+	}
+
+	ret = sr_scpi_cmd_resp(sdi, devc->device->commands,
+		channel_group_cmd, channel_group_name, data, gvtype, cmd);
+	g_free(channel_group_name);
 
 	if (cmd == SCPI_CMD_GET_OUTPUT_REGULATION) {
 		/*
@@ -440,126 +444,131 @@ static int config_get(uint32_t key, GVariant **data, const struct sr_dev_inst *s
 	return ret;
 }
 
-static int config_set(uint32_t key, GVariant *data, const struct sr_dev_inst *sdi,
-		const struct sr_channel_group *cg)
+static int config_set(uint32_t key, GVariant *data,
+	const struct sr_dev_inst *sdi, const struct sr_channel_group *cg)
 {
 	struct dev_context *devc;
 	double d;
+	int channel_group_cmd;
+	char *channel_group_name;
 	int ret;
 
 	if (!sdi)
 		return SR_ERR_ARG;
 
-	if (sdi->status != SR_ST_ACTIVE)
-		return SR_ERR_DEV_CLOSED;
-
-	if (cg)
-		/* Channel group specified. */
-		select_channel(sdi, cg->channels->data);
+	channel_group_cmd = 0;
+	channel_group_name = NULL;
+	if (cg) {
+		channel_group_cmd = SCPI_CMD_SELECT_CHANNEL;
+		channel_group_name = g_strdup(cg->name);
+	}
 
 	devc = sdi->priv;
 
 	switch (key) {
 	case SR_CONF_ENABLED:
 		if (g_variant_get_boolean(data))
-			ret = scpi_cmd(sdi, devc->device->commands,
+			ret = sr_scpi_cmd(sdi, devc->device->commands,
+					channel_group_cmd, channel_group_name,
 					SCPI_CMD_SET_OUTPUT_ENABLE);
 		else
-			ret = scpi_cmd(sdi, devc->device->commands,
+			ret = sr_scpi_cmd(sdi, devc->device->commands,
+					channel_group_cmd, channel_group_name,
 					SCPI_CMD_SET_OUTPUT_DISABLE);
 		break;
 	case SR_CONF_VOLTAGE_TARGET:
 		d = g_variant_get_double(data);
-		ret = scpi_cmd(sdi, devc->device->commands,
+		ret = sr_scpi_cmd(sdi, devc->device->commands,
+				channel_group_cmd, channel_group_name,
 				SCPI_CMD_SET_VOLTAGE_TARGET, d);
 		break;
 	case SR_CONF_OUTPUT_FREQUENCY_TARGET:
 		d = g_variant_get_double(data);
-		ret = scpi_cmd(sdi, devc->device->commands,
+		ret = sr_scpi_cmd(sdi, devc->device->commands,
+				channel_group_cmd, channel_group_name,
 				SCPI_CMD_SET_FREQUENCY_TARGET, d);
 		break;
 	case SR_CONF_CURRENT_LIMIT:
 		d = g_variant_get_double(data);
-		ret = scpi_cmd(sdi, devc->device->commands,
+		ret = sr_scpi_cmd(sdi, devc->device->commands,
+				channel_group_cmd, channel_group_name,
 				SCPI_CMD_SET_CURRENT_LIMIT, d);
 		break;
 	case SR_CONF_OVER_VOLTAGE_PROTECTION_ENABLED:
 		if (g_variant_get_boolean(data))
-			ret = scpi_cmd(sdi, devc->device->commands,
+			ret = sr_scpi_cmd(sdi, devc->device->commands,
+					channel_group_cmd, channel_group_name,
 					SCPI_CMD_SET_OVER_VOLTAGE_PROTECTION_ENABLE);
 		else
-			ret = scpi_cmd(sdi, devc->device->commands,
+			ret = sr_scpi_cmd(sdi, devc->device->commands,
+					channel_group_cmd, channel_group_name,
 					SCPI_CMD_SET_OVER_VOLTAGE_PROTECTION_DISABLE);
 		break;
 	case SR_CONF_OVER_VOLTAGE_PROTECTION_THRESHOLD:
 		d = g_variant_get_double(data);
-		ret = scpi_cmd(sdi, devc->device->commands,
+		ret = sr_scpi_cmd(sdi, devc->device->commands,
+				channel_group_cmd, channel_group_name,
 				SCPI_CMD_SET_OVER_VOLTAGE_PROTECTION_THRESHOLD, d);
 		break;
 	case SR_CONF_OVER_CURRENT_PROTECTION_ENABLED:
 		if (g_variant_get_boolean(data))
-			ret = scpi_cmd(sdi, devc->device->commands,
+			ret = sr_scpi_cmd(sdi, devc->device->commands,
+					channel_group_cmd, channel_group_name,
 					SCPI_CMD_SET_OVER_CURRENT_PROTECTION_ENABLE);
 		else
-			ret = scpi_cmd(sdi, devc->device->commands,
+			ret = sr_scpi_cmd(sdi, devc->device->commands,
+					channel_group_cmd, channel_group_name,
 					SCPI_CMD_SET_OVER_CURRENT_PROTECTION_DISABLE);
 		break;
 	case SR_CONF_OVER_CURRENT_PROTECTION_THRESHOLD:
 		d = g_variant_get_double(data);
-		ret = scpi_cmd(sdi, devc->device->commands,
+		ret = sr_scpi_cmd(sdi, devc->device->commands,
+				channel_group_cmd, channel_group_name,
 				SCPI_CMD_SET_OVER_CURRENT_PROTECTION_THRESHOLD, d);
 		break;
 	case SR_CONF_OVER_TEMPERATURE_PROTECTION:
 		if (g_variant_get_boolean(data))
-			ret = scpi_cmd(sdi, devc->device->commands,
+			ret = sr_scpi_cmd(sdi, devc->device->commands,
+					channel_group_cmd, channel_group_name,
 					SCPI_CMD_SET_OVER_TEMPERATURE_PROTECTION_ENABLE);
 		else
-			ret = scpi_cmd(sdi, devc->device->commands,
+			ret = sr_scpi_cmd(sdi, devc->device->commands,
+					channel_group_cmd, channel_group_name,
 					SCPI_CMD_SET_OVER_TEMPERATURE_PROTECTION_DISABLE);
 		break;
 	default:
-		ret = SR_ERR_NA;
+		ret = sr_sw_limits_config_set(&devc->limits, key, data);
 	}
 
+	g_free(channel_group_name);
+
 	return ret;
 }
 
-static int config_list(uint32_t key, GVariant **data, const struct sr_dev_inst *sdi,
-		const struct sr_channel_group *cg)
+static int config_list(uint32_t key, GVariant **data,
+	const struct sr_dev_inst *sdi, const struct sr_channel_group *cg)
 {
 	struct dev_context *devc;
 	struct sr_channel *ch;
 	const struct channel_spec *ch_spec;
-	GVariant *gvar;
-	GVariantBuilder gvb;
-	int ret, i;
+	int i;
 	const char *s[16];
 
-	/* Always available, even without sdi. */
-	if (key == SR_CONF_SCAN_OPTIONS) {
-		*data = g_variant_new_fixed_array(G_VARIANT_TYPE_UINT32,
-				scanopts, ARRAY_SIZE(scanopts), sizeof(uint32_t));
-		return SR_OK;
-	} else if (key == SR_CONF_DEVICE_OPTIONS && !sdi) {
-		*data = g_variant_new_fixed_array(G_VARIANT_TYPE_UINT32,
-				drvopts, ARRAY_SIZE(drvopts), sizeof(uint32_t));
-		return SR_OK;
-	}
+	devc = (sdi) ? sdi->priv : NULL;
 
-	if (!sdi)
-		return SR_ERR_ARG;
-	devc = sdi->priv;
-
-	ret = SR_OK;
 	if (!cg) {
-		/* No channel group: global options. */
 		switch (key) {
+		case SR_CONF_SCAN_OPTIONS:
 		case SR_CONF_DEVICE_OPTIONS:
-			*data = g_variant_new_fixed_array(G_VARIANT_TYPE_UINT32,
-					devc->device->devopts, devc->device->num_devopts,
-					sizeof(uint32_t));
+			return std_opts_config_list(key, data, sdi, cg,
+				ARRAY_AND_SIZE(scanopts),
+				ARRAY_AND_SIZE(drvopts),
+				(devc && devc->device) ? devc->device->devopts : NULL,
+				(devc && devc->device) ? devc->device->num_devopts : 0);
 			break;
 		case SR_CONF_CHANNEL_CONFIG:
+			if (!devc || !devc->device)
+				return SR_ERR_ARG;
 			/* Not used. */
 			i = 0;
 			if (devc->device->features & PPS_INDEPENDENT)
@@ -581,7 +590,6 @@ static int config_list(uint32_t key, GVariant **data, const struct sr_dev_inst *
 			return SR_ERR_NA;
 		}
 	} else {
-		/* Channel group specified. */
 		/*
 		 * Per-channel-group options depending on a channel are actually
 		 * done with the first channel. Channel groups in PPS can have
@@ -589,86 +597,54 @@ static int config_list(uint32_t key, GVariant **data, const struct sr_dev_inst *
 		 * specification for use in series or parallel mode.
 		 */
 		ch = cg->channels->data;
+		if (!devc || !devc->device)
+			return SR_ERR_ARG;
+		ch_spec = &(devc->device->channels[ch->index]);
 
 		switch (key) {
 		case SR_CONF_DEVICE_OPTIONS:
-			*data = g_variant_new_fixed_array(G_VARIANT_TYPE_UINT32,
-					devc->device->devopts_cg, devc->device->num_devopts_cg,
-					sizeof(uint32_t));
+			*data = std_gvar_array_u32(devc->device->devopts_cg, devc->device->num_devopts_cg);
 			break;
 		case SR_CONF_VOLTAGE_TARGET:
-			ch_spec = &(devc->device->channels[ch->index]);
-			g_variant_builder_init(&gvb, G_VARIANT_TYPE_ARRAY);
-			/* Min, max, write resolution. */
-			for (i = 0; i < 3; i++) {
-				gvar = g_variant_new_double(ch_spec->voltage[i]);
-				g_variant_builder_add_value(&gvb, gvar);
-			}
-			*data = g_variant_builder_end(&gvb);
+			*data = std_gvar_min_max_step_array(ch_spec->voltage);
 			break;
 		case SR_CONF_OUTPUT_FREQUENCY_TARGET:
-			ch_spec = &(devc->device->channels[ch->index]);
-			g_variant_builder_init(&gvb, G_VARIANT_TYPE_ARRAY);
-			/* Min, max, write resolution. */
-			for (i = 0; i < 3; i++) {
-				gvar = g_variant_new_double(ch_spec->frequency[i]);
-				g_variant_builder_add_value(&gvb, gvar);
-			}
-			*data = g_variant_builder_end(&gvb);
+			*data = std_gvar_min_max_step_array(ch_spec->frequency);
 			break;
 		case SR_CONF_CURRENT_LIMIT:
-			g_variant_builder_init(&gvb, G_VARIANT_TYPE_ARRAY);
-			/* Min, max, step. */
-			for (i = 0; i < 3; i++) {
-				ch_spec = &(devc->device->channels[ch->index]);
-				gvar = g_variant_new_double(ch_spec->current[i]);
-				g_variant_builder_add_value(&gvb, gvar);
-			}
-			*data = g_variant_builder_end(&gvb);
+			*data = std_gvar_min_max_step_array(ch_spec->current);
+			break;
+		case SR_CONF_OVER_VOLTAGE_PROTECTION_THRESHOLD:
+			*data = std_gvar_min_max_step_array(ch_spec->ovp);
+			break;
+		case SR_CONF_OVER_CURRENT_PROTECTION_THRESHOLD:
+			*data = std_gvar_min_max_step_array(ch_spec->ocp);
 			break;
 		default:
 			return SR_ERR_NA;
 		}
 	}
 
-	return ret;
+	return SR_OK;
 }
 
 static int dev_acquisition_start(const struct sr_dev_inst *sdi)
 {
 	struct dev_context *devc;
 	struct sr_scpi_dev_inst *scpi;
-	struct sr_channel *ch;
-	struct pps_channel *pch;
-	int cmd, ret;
-
-	if (sdi->status != SR_ST_ACTIVE)
-		return SR_ERR_DEV_CLOSED;
+	int ret;
 
 	devc = sdi->priv;
 	scpi = sdi->conn;
 
+	/* Prime the pipe with the first channel. */
+	devc->cur_acquisition_channel = sr_next_enabled_channel(sdi, NULL);
+
 	if ((ret = sr_scpi_source_add(sdi->session, scpi, G_IO_IN, 10,
 			scpi_pps_receive_data, (void *)sdi)) != SR_OK)
 		return ret;
 	std_session_send_df_header(sdi);
-
-	/* Prime the pipe with the first channel's fetch. */
-	ch = sr_next_enabled_channel(sdi, NULL);
-	pch = ch->priv;
-	if ((ret = select_channel(sdi, ch)) < 0)
-		return ret;
-	if (pch->mq == SR_MQ_VOLTAGE)
-		cmd = SCPI_CMD_GET_MEAS_VOLTAGE;
-	else if (pch->mq == SR_MQ_FREQUENCY)
-		cmd = SCPI_CMD_GET_MEAS_FREQUENCY;
-	else if (pch->mq == SR_MQ_CURRENT)
-		cmd = SCPI_CMD_GET_MEAS_CURRENT;
-	else if (pch->mq == SR_MQ_POWER)
-		cmd = SCPI_CMD_GET_MEAS_POWER;
-	else
-		return SR_ERR;
-	scpi_cmd(sdi, devc->device->commands, cmd, pch->hwname);
+	sr_sw_limits_acquisition_start(&devc->limits);
 
 	return SR_OK;
 }
@@ -676,19 +652,9 @@ static int dev_acquisition_start(const struct sr_dev_inst *sdi)
 static int dev_acquisition_stop(struct sr_dev_inst *sdi)
 {
 	struct sr_scpi_dev_inst *scpi;
-	float f;
-
-	if (sdi->status != SR_ST_ACTIVE)
-		return SR_ERR_DEV_CLOSED;
 
 	scpi = sdi->conn;
 
-	/*
-	 * A requested value is certainly on the way. Retrieve it now,
-	 * to avoid leaving the device in a state where it's not expecting
-	 * commands.
-	 */
-	sr_scpi_get_float(scpi, NULL, &f);
 	sr_scpi_source_remove(sdi->session, scpi);
 
 	std_session_send_df_end(sdi);
